;+
;Procedure: enp_crib.pro
;
;Purpose:  A crib showing how to transform data into the ENP and PEN coordinate systems. 
;
;   E: sat to earth (in-plane)
;   N: east (in-plane)
;   P: north (perpendicular to plane).
;
;  Defined relative to another coordinate system:
;   P_sat = spacecraft position in geocentric interial coordinate system
;   V_sat = deriv(P_sat)   (spacecraft velocity in the same coordinate system.)
;
;   P_enp = P_sat cross V_sat
;   E_enp = -P_sat
;   N_enp = P_enp cross P_sat
;
;Notes:
;
; $LastChangedBy: pcruce $
; $LastChangedDate: 2009-09-16 15:18:08 -0700 (Wed, 16 Sep 2009) $
; $LastChangedRevision: 6741 $
; $URL: svn+ssh://thmsvn@ambrosia.ssl.berkeley.edu/repos/ssl_general/trunk/cotrans/special/enp/enp_crib.pro $
;-



  ;set the time
  timespan,'2008-03-23'

  thm_load_goesmag,probe='g10'

  ;----------------
  ;ENP system
  ;----------------
  
  ;input position MUST be earth-centered and inertial
  ;velocity/orbital plane determined using derivative of position.
  ;If there are not enough points, the result will be inaccurate.  Also, may have small error at the tails.
  enp_matrix_make,'g10_pos_gei'  ;accepts newname,suffix keywords, can use globbing on inputs

  ;--------------------
  ;GEI->ENP
  ;--------------------

  ;this routine interpolate rotation matrices automatically, no need to match cadence ahead of time
  tvector_rotate,'g10_pos_gei_enp_mat','g10_b_gei'   ;accepts newname,suffix keywords, can use globbing on inputs

  options,'g10_b_gei_rot',labels=['E','N','P'],ytitle='g10_b_enp'

  tplot,['g10_b_gei_rot','g10_b_enp'] ;should match
  
  stop
  
  ;--------------------
  ;ENP->GEI
  ;-------------------- 
 
  tvector_rotate,'g10_pos_gei_enp_mat','g10_b_enp',/invert  ;invert keyword runs transformation backwards

  options,'g10_b_enp_rot',labels=['Bx','By','Bz'],ytitle='g10_b_gei'

  tplot,['g10_b_enp_rot','g10_b_gei'] ;should match
  
  stop
  
  ;----------------
  ;PEN system
  ;----------------
  
  ;input position MUST be earth-centered and inertial
  enp_matrix_make,'g10_pos_gei',/pen  ;accepts newname,suffix keywords, can use globbing on inputs
  
  ;--------------------
  ;GEI->PEN
  ;--------------------

  ;this routine interpolate rotation matrices automatically, no need to match cadence ahead of time
  tvector_rotate,'g10_pos_gei_pen_mat','g10_b_gei',newname='g10_b_pen'   ;accepts newname,suffix keywords, can use globbing on inputs

  options,'g10_b_gei_rot',labels=['P','E','N'],ytitle='g10_b_pen'

  tplot,['g10_b_pen','g10_b_enp'] ; shouldn't match, the axes are permuted
  
  stop
  
  ;--------------------
  ;PEN->GEI
  ;-------------------- 
 
  tvector_rotate,'g10_pos_gei_pen_mat','g10_b_pen',/invert ;invert keyword runs transformation backwards

  options,'g10_b_pen_rot',labels=['Bx','By','Bz'],ytitle='g10_b_gei'

  tplot,['g10_b_pen_rot','g10_b_gei'] ;should match
  
  stop
  
  ;-----------------------
  ;User Generated Velocity
  ;-----------------------
  
  tinterpol_mxn,'g10_pos_gei','g10_b_enp'
  
  enp_matrix_make,'g10_pos_gei',velocity_tvar='g10_vel_gei'
  
  tvector_rotate,'g10_pos_gei_enp_mat','g10_b_gei'   ;accepts newname,suffix keywords, can use globbing on inputs

  options,'g10_b_gei_rot',labels=['E','N','P'],ytitle='g10_b_enp'

  tplot,['g10_b_gei_rot','g10_b_enp'] ;should match
  
  stop
end