;+
;Procedure: enp_matrix_make
;
;Purpose: 
;  Creates a set of matrices that will rotate data from the input coordinate system into
;  ENP.
;   E: sat to earth (in-plane)
;   N: east (in-plane)
;   P: north (perpendicular to plane).
;
;  Defined relative to another coordinate system:
;   P_sat = spacecraft position in geocentric interial coordinate system
;   V_sat = deriv(P_sat)   (spacecraft velocity in the same coordinate system.)
;
;   P_enp = P_sat cross V_sat
;   E_enp = -P_sat
;   N_enp = P_enp cross P_sat
;   
;Inputs:
;  pos_tvarname:  Tplot Variable storing the spacecraft position in an intertial earth-centered cartesian coordinate system(like gei)
;                 You can use globbing to pass multiple position vectors simultaneously. ('th?_state_pos')  
;  
;Outputs: 
;  xxx2enp transformation matrix
;  
; Keywords:
;   /pen  : reorders the basis vectors so that they transform to and from PEN rather than ENP
;   fail=fail: Will be set to 1 if operation failed, returns 0 if operation succeeded.  Will not signal failure if
;        at least one input was processed.
;  suffix: The suffix to be appended to the tplot variables that the output matrices will be stored in.
;         (Default: '_enp_mat' or '_pen_mat')
;  newname: The name of the output matrix.  If this keyword is used with multiple input values, the outputs
;          may overwrite each other.  So you should only set this keyword if there is a single value for the state input.
;  velocity_tvar:  Set this keyword to the name of a tplot variable or variables that store velocities matching the positions.
;           This way the routine doesn't need to calculate the velocity using derivatives  
;   
;Notes:
;  Because velocity is calculated using the derivative of position,
;  there should be a small numerical error at the end points of the time series
;
; $LastChangedBy: pcruce $
; $LastChangedDate: 2009-09-16 12:37:46 -0700 (Wed, 16 Sep 2009) $
; $LastChangedRevision: 6739 $
; $URL: svn+ssh://thmsvn@ambrosia.ssl.berkeley.edu/repos/ssl_general/trunk/cotrans/special/enp/enp_matrix_make.pro $
;-



pro enp_matrix_make,pos_tvarname,fail=fail,pen=pen,suffix=suffix,newname=newname,velocity_tvar=velocity_tvar

  compile_opt idl2
  
  fail = 1
  
  if ~keyword_set(pos_tvarname) then begin
    dprint,'pos_tvarname not set'
    return
  endif
  
  varnames = tnames(pos_tvarname)
  
  if varnames[0] eq '' then begin
    dprint,'Failed, no tvars match pos_tvarname'
    return
  endif
  
  if keyword_set(velocity_tvar) then begin
    velnames = tnames(velocity_tvar)
    
    if velnames[0] eq '' then begin
      dprint,'Failed, velocity_tvar is invalid tplot variable.'
      return
    endif
    
    if n_elements(velnames) ne n_elements(varnames) then begin
      dprint,'Failed, number of position tvars and velocity tvars does not match'
      return
    endif
  
  endif
  
  if keyword_set(pen) then begin
    coord_sys = 'pen'
  endif else begin
    coord_sys = 'enp'
  endelse
  
  if ~keyword_set(suffix) then begin
    suffix = '_'+coord_sys+'_mat' 
  endif
  
  for i = 0,n_elements(varnames)-1 do begin
  
    undefine,vdata
  
    source_sys = cotrans_get_coord(varnames[i])
    
    get_data,varnames[i],data=d,dlimits=dl,limits=l
    
    if ~is_struct(d) then begin
      dprint,varnames[i] + ' does not have a valid data component.  Skipping variable.'
      continue
    endif
  
    if ~in_set(strlowcase(tag_names(d)),'y') then begin
      dprint,varnames[i] + ' does not have a valid data component.  Skipping variable.'
      continue
    endif
    
    if ~in_set(strlowcase(tag_names(d)),'x') then begin
      dprint,varnames[i] + ' does not have a valid time component.  Skipping variable.'
      continue
    endif
  
    if in_set(strlowcase(tag_names(d)),'v') then begin
      vdata = d.v
    endif
    
    tdata = d.x
    
    dim = dimen(d.y)
    
    dData = double(d.y)
    
    undefine,d      
  
    if n_elements(dim) ne 2 || dim[1] ne 3 || dim[0] eq 1 then begin
      dprint,varnames[i] + ' data component has incorrect dimensions. Skipping variable.'
      continue
    endif
    
    if ~keyword_set(velnames) then begin
      velData = dblarr(dim)
    
      velData[*,0] = deriv(tData,dData[*,0])
      velData[*,1] = deriv(tData,dData[*,1])
      velData[*,2] = deriv(tData,dData[*,2])
    endif else begin
    
      tinterpol_mxn,velnames[i],varnames[i],out=d
      
      if ~is_struct(d) then begin
        dprint,velnames[i] + ' does not have a valid data component.  Skipping variable.'
        continue
      endif
    
      if ~in_set(strlowcase(tag_names(d)),'y') then begin
        dprint,velnames[i] + ' does not have a valid data component.  Skipping variable.'
        continue
      endif

      velData = d.y
     
    endelse
      
    tcrossp,dData,velData,out=pData
    eData = -dData
    tcrossp,pData,dData,out=nData
    
    ;waits till the end to normalize, cross products produce reliable directions indepedent of vector magnitude
    tnormalize,pData,out=pUnit
    tnormalize,eData,out=eUnit
    tnormalize,nData,out=nUnit
    
    out = dblarr(dim[0],3,3)
    
    if coord_sys eq 'pen' then begin
      out[*,0,*] = pUnit
      out[*,1,*] = eUnit
      out[*,2,*] = nUnit
    endif else begin
      out[*,0,*] = eUnit
      out[*,1,*] = nUnit
      out[*,2,*] = pUnit
    endelse
    
    if n_elements(vData) gt 0 then begin
      outd = {x:temporary(tData),y:temporary(out),v:temporary(vData)}
    endif else begin
      outd = {x:temporary(tData),y:temporary(out)}
    endelse
   
    ;tvector rotate uses this flag to determine if it should take axes
    ;labels from the transformation matrix
   
    str_element,dl,'labflag',value=0,/add_replace
    str_element,dl,'data_att.coord_sys',coord_sys,/add_replace
    str_element,dl,'data_att.source_sys',source_sys,/add_replace
    
    if ~keyword_set(newname) then begin
      store_data,varnames[i]+suffix,data=outd,dlimits=dl,limits=l
    endif else begin
      store_data,newname,data=outd,dlimits=dl,limits=l
    endelse
  
    success=1
    
  endfor
  
  ;if at least one output is processed,
  ;then set fail = 0
  if keyword_set(success) then begin
    fail = 0
  endif
  
end